/*
 * File         : Reactor.cpp
 * Author       : Tomasz Stachowiak
 * Created      : Wed May 20 14:55:02 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

// STL
#include <vector.h>

#include <BVN/BVN.h>
#include <BVN/Event_Handler.h>
#include <BVN/Reactor.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>


Reactor::Reactor(void)
{
  first = 1;
  acc_handle = 0;
}

int Reactor::open(void)
{
  return 0;
}

int Reactor::close(void)
{
  vector <Handler>::iterator i = handlers.begin();
  while(i != handlers.end())
    {
      ::close((*i).h);
      delete (*i).eh;
      handlers.erase(i);
      i = handlers.begin();
    }

  return 0;
}

int Reactor::reset()
{
  vector <Handler>::iterator i = handlers.begin();
  while(i != handlers.end())
    {
      if((*i).h != acc_handle)
	{
	  ::close((*i).h);
	  delete (*i).eh;
	  handlers.erase(i);
	  i = handlers.begin();
	}
      else
	i++;
    }

  return 0;  
}

int Reactor::handle_events (int max_wait_time)
{
  fd_set set;
  struct timeval timeout;
  int done = 0;
  int err;

  //BVN_DEBUG("Reactor::handle_events\n");

  timeout.tv_sec = max_wait_time / 1000;
  timeout.tv_usec = (max_wait_time % 1000) * 1000;
  
  FD_ZERO(&set);

  vector <Handler>::iterator i = handlers.begin();
  while(i != handlers.end())
    {
      //BVN_DEBUG("\tset handler %d\n", (*i).h);

      FD_SET((*i).h, &set);
      i++;
    }

  //BVN_DEBUG("\tselect\n");
  err = select(FD_SETSIZE, &set, 0, 0, &timeout);
  if(err > 0)
    {
      //BVN_DEBUG("\thandle active\n");
      i = handlers.begin();
      while((i != handlers.end()) && (done == 0))
	{
	  //BVN_DEBUG("\tcheck if set %d\n",(*i).h );
	  if(FD_ISSET((*i).h, &set))
	  {	  
	    //BVN_DEBUG("\tset\n");
	    (*i).eh->handle_event(READ_EVENT);
	    done = 1;
	  }
	  i++;
	}
      
      
    }
  if(err < 0)
    return -1;

  return 0;
  
}
  
int Reactor::register_handler(Event_Handler *handler, Event_Type type)
{
  Handler hdl;

  BVN_DEBUG("Reactor::register_handler\n");

  hdl.eh = handler;
  hdl.t = type;
  hdl.h = handler->get_handle();
  handlers.insert(handlers.end(), hdl);
  if(first == 1)
    {
      first = 0;
      acc_handle = hdl.h;
    }

  BVN_DEBUG("Reactor:: handler %d registered\n", hdl.h);
  return 0;
}

int Reactor::remove_handler(Event_Handler *handler, Event_Type type)
{
  int done = 0;

  BVN_DEBUG("Reactor::remove_handler\n");

  vector <Handler>::iterator i = handlers.begin();
  while((i != handlers.end()) && (done == 0))
    {
      if(((*i).eh == handler) && ((*i).t == type))
	{
	  delete (*i).eh;
	  handlers.erase(i);
	  done = 1;
	}
      i++;
    }

  return 0;
}
